import json
from json.decoder import JSONDecodeError
from typing import Dict
from urllib.parse import urlencode

import requests
from bs4 import BeautifulSoup

try:
    from utils import udict
except (ModuleNotFoundError, ImportError) as e:
    from pysubway.utils import udict


class HuaDaoBase:
    """
    华道返回 xml
    """
    url_access_token = 'http://opensdk.emay.cn:9080/HD_GetAccess_Token.asmx/GetACCESS_TOKEN'

    def __init__(self, token: str):
        self.token: str = token
        self.full_url: str = ''
        self.biz_data: Dict = dict()

    @classmethod
    def get_token(cls, AppID: str, AppSecret: str, Key: str) -> str:
        full_url = '?'.join((cls.url_access_token, urlencode({
            'AppID': AppID,
            'AppSecret': AppSecret,
            'Key': Key
        })))
        r = requests.get(full_url)
        return r.text

    @staticmethod
    def generate_full_url(url: str, biz_data: Dict[str, str]) -> str:
        """
        you can get full_url by this function return
        :param url: base url
        :param biz_data:
        :return:
        """
        return '?'.join((url, urlencode(biz_data)))

    def set_full_url(self, url: str, biz_data: Dict[str, str]) -> 'HuaDaoBase':
        self.full_url = self.generate_full_url(url, biz_data)
        return self

    def request(self, full_url: str = '') -> Dict[str, str]:
        full_url = full_url or self.full_url
        r = requests.get(full_url)
        if r.ok:
            try:
                bs = BeautifulSoup(r.text, features='html.parser')
                return json.loads(bs.text)
            except JSONDecodeError as e:
                raise TypeError(f'JSONDecodeError, raw resp {r.text}')
            except Exception as e:
                raise SystemError(f'raw resp1 {r.text}')
        else:
            raise ValueError(f'raw resp2 {r.text}')


class BankFiveElements(HuaDaoBase):
    url = 'http://opensdk.emay.cn:9099/SF_YZ_API/SFService.asmx/Get_EMW_GetBank_Card_Five_RZ'

    def set_biz_data(self, name: str = '', idcard: str = '', cardNo: str = '', phone: str = '',
                     bankAccountType: str = '') -> 'BankFiveElements':
        self.biz_data = udict.Dict.filter(locals())
        self.biz_data['ACCESS_TOKEN'] = self.token
        return self



import json
import typing

try:
    from utils.container import is_iterable
except (ModuleNotFoundError, ImportError) as e:
    from pysubway.utils.container import is_iterable


class Dict(dict):

    def sort_keys(self, sort_by_ascii: bool = True) -> str:
        """
        sort dict value by dict key's order in ascii.
        :param data_dict:
        :return:
        """
        if sort_by_ascii:
            sign_data = ''
            for key in sorted(self.keys()):
                value = self[key]
                if isinstance(value, (dict, list)):
                    sign_data += json.dumps(self[key], ensure_ascii=False, sort_keys=True)
                else:
                    sign_data += str(value)
            return sign_data
        else:
            raise NotImplementedError(f'sort_by_ascii {sort_by_ascii}')

    @staticmethod
    def filter(data: typing.Dict[str, typing.Any], exclude: typing.Tuple[str, ...] = ('self', 'cls'),
               forbid_empty_val: bool = True) -> typing.Dict[
        str, typing.Any]:
        if forbid_empty_val and '' in data.items():
            raise ValueError(f'empty value existed in {data}')
        return {k: v for k, v in data.items() if k not in exclude}

    def remove(self, data: typing.Union[typing.Iterable, str]) -> 'Dict':
        """
        remove data and return instance
        :param data:
        :return:
        """
        data = data if is_iterable(data) else (data,)
        for k in data:
            self.pop(k, None)
        return self


if __name__ == '__main__':
    w = Dict(a=1, b=1)
    m = w.pop('a')
    print(m)



import io
import os
import tempfile
from typing import TextIO, Union, Tuple

import chardet
from chardet.gb2312prober import GB2312Prober
from chardet.latin1prober import Latin1Prober

try:
    from utils.ustring import replace_chinese_comma
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.utils.ustring import replace_chinese_comma


def file_dirname(file: str) -> str:
    return os.path.split(file)[0]


def join_path(dirname: str, *path: str) -> str:
    return os.path.join(dirname, *path)


class File:
    file_dirname = staticmethod(file_dirname)
    join_path = staticmethod(join_path)

    gdk_compatibility = [
        # ascii
        Latin1Prober().charset_name,
        # gbk
        GB2312Prober().charset_name,
        'ISO-8859-9',
    ]

    def __init__(self, file: Union[str, TextIO]):
        self.file: Union[str, TextIO] = file
        self.dirname, self.filename = os.path.split(self.file) if isinstance(self.file, str) else ('', '')
        self.pure_name, self.suffix = self.get_pure_name_and_suffix(self.filename)
        self._encoding: str = ''

    @staticmethod
    def get_pure_name_and_suffix(filename: str) -> Tuple[str, str]:
        split = filename.split('.')
        if len(split) == 1:
            split.append('')
        return split[0], split[-1]

    @property
    def encoding(self) -> str:
        """
        {'encoding': 'ascii', 'confidence': 1.0, 'language': ''}
        :param self:
        :return:
        """
        if not self._encoding:
            if isinstance(self.file, str):
                with open(self.file, mode='rb') as f:
                    # must read all, or the detect result is imprecision
                    encoding = chardet.detect(f.read())
                    self._encoding = 'gbk' if encoding in self.gdk_compatibility else 'utf-8'
        return self._encoding

    def remove_chinese_comma(self, output: str = 'io') -> Union[str, TextIO, io.StringIO]:
        f: TextIO = io.StringIO()
        try:
            # if not str, treat self.file as TextIO
            f = open(self.file) if isinstance(self.file, str) else self.file
            s = replace_chinese_comma(f.read())
            if output != 'io':
                temp = tempfile.NamedTemporaryFile(mode='w', delete=False, encoding=self.encoding)
                temp.write(s)
                temp.close()
                return temp.name
            else:
                temp = io.StringIO()
                temp.write(s)
                temp.seek(0)
                return temp
        finally:
            f.close()

    @staticmethod
    def this_dir(path: str) -> str:
        """
        return current file dirname
        you can call this method like this_dir(__file__)
        :return: str
        """
        return os.path.dirname(os.path.abspath(path))


class Open(File):
    """
    example:
    >>>with File(__file__) as f:
    >>> for i in range(10):
    >>>     print(f.readline())
    """

    def __init__(self, file: Union[str, TextIO]):
        super().__init__(file)
        self.f = self.file if isinstance(self.file, (io.StringIO, TextIO)) else open(self.file, encoding=self.encoding)

    def __enter__(self) -> TextIO:
        return self.f

    def __exit__(self, exc_type: str, exc_val: str, exc_tb: str) -> None:
        self.f.close()



from typing import Dict, Optional, Tuple, Any, Union, TextIO, List

import pandas as pd

try:
    from service.booldata.base import BoolDataBase
    from utils import udict
    from utils.uexcel import Excel
    from utils.ujson import Json2Excel
    from utils.file import File
    from errors import RequestFailed
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.service.booldata.base import BoolDataBase
    from pysubway.utils import udict
    from pysubway.utils.uexcel import Excel
    from pysubway.utils.ujson import Json2Excel
    from pysubway.utils.file import File
    from pysubway.errors import RequestFailed


class HzRiskModel(BoolDataBase):
    service = 'hz_risk_model_service'
    mode = 'hz_risk_model_personal_credit_mode'
    df_default_header = ('姓名',
                         '年龄',
                         '身份证号',
                         '审核建议',
                         '3个月手机号关联身份证数', '3个月身份证关联手机号数', '个人欺诈黑名单',
                         '信贷逾期名单', '助学贷款欠费历史', '命中风险标注',
                         '户籍', '户籍地位于高风险集中地区', '手机号', '手机号归属地',
                         '故意违章乘车名单', '是否结清', '最近一次放款日期', '最近一次申请日期',
                         '欠税公司法人代表名单', '欠税名单', '法院信息', '法院失信名单',
                         '法院执行名单', '法院结案名单', '消费分期类放款机构数', '消费分期类申请机构数',
                         '犯罪通缉名单', '综合评分', '网络贷款类放款机构数', '网络贷款类申请机构数',
                         '虚拟号码库', '距离最近一次申请日期天数', '车辆租赁违约名单',
                         '近12个月履约次数', '近12个月放款机构总数', '近12个月申请机构总数',
                         '近12个月申请次数', '近12个月还款异常次数', '近1个月履约次数', '近1个月放款次数',
                         '近1个月申请次数', '近1个月还款异常次数', '近30天P2P平台', '近30天一般消费分期平台',
                         '近30天信用卡', '近30天其他类型公司', '近30天大型消费金融公司',
                         '近30天小额贷款公司', '近30天申请查询总数', '近3个月放款次数', '近3个月申请次数',
                         '近6个月放款次数', '近6个月未结清逾期次数', '近6个月申请次数',
                         '近6个月逾期总次数', '近6个月逾期总金额（元）', '近6个月逾期机构总数',
                         '近7天P2P平台', '近7天一般消费分期平台', '近7天信用卡', '近7天其他类型公司',
                         '近7天大型消费金融公司', '近7天小额贷款公司', '近7天申请查询总数', '近90天P2P平台',
                         '近90天一般消费分期平台', '近90天信用卡', '近90天其他类型公司',
                         '近90天大型消费金融公司', '近90天小额贷款公司', '近90天申请查询总数',
                         '通信小号库', '逾期时长', '逾期时间', '逾期金额区间（元）', '高风险关注名单')

    @classmethod
    def generate_biz_data(cls, name: str, phone: str, ident_number: str) -> Dict[str, Any]:
        biz_data = udict.Dict.filter(locals())
        biz_data.update({
            "service": cls.service,
            "mode": cls.mode
        })
        import pandas as pd
        pd.Series({'a': 1, 'b': 2, 'c': 3, }, index=['c', 'b', 'a', 'f'])
        return biz_data

    def set_biz_data(self, name: str, phone: str, ident_number: str) -> 'HzRiskModel':
        self.biz_data = self.generate_biz_data(name, phone, ident_number)
        return self


def batch_test(
        input: Union[str, pd.DataFrame, TextIO],
        account: Dict[str, str],
        output_file: str,
        template: str = f'{File.this_dir(__file__)}/template_hz_risk_model.xlsx',
        names: Optional[Tuple[str, ...]] = ('name', 'phone', 'idcard'),
        columns: Tuple[str, ...] = HzRiskModel.df_default_header,
        skiprows: Union[int, Tuple[int, ...], None] = None,
        error_log: Optional[str] = None,
        header: Union[int, List[int], None] = 0,
        raise_failed_exception: bool = False,
        env: str = 'prod',
) -> None:
    """
    batch test HzRiskModel, read from excel -> request HzRiskModel -> save result to excel

    :param template: result is transferred excel as the template
    :param input: excel file or pd.DataFrame or TextIO. If input not str, you must specify param error_log
    :param account: booldata account
    :param output_file: xlsx file that saving booldata result
    :param names: input excel file header, you can adjust the sequence.
    :param columns: output_file columns header
    :param skiprows: strip rows
    :param error_log: records error biz_data item.
    :param header: input excel header, default first row in excel
    :param raise_failed_exception: if True, raise RequestFailed Exception when request booldata failed.
    :return: None
    """
    discard_params = ['', None]
    error_log = error_log or File.join_path(File(input).dirname, '.'.join((File(input).pure_name, 'error_log')))
    template_excel = Json2Excel(template)
    input_excel: pd.DataFrame = Excel(input).read(names=names, skiprows=skiprows,
                                                  header=header).dataframe if isinstance(input, str) else input
    df_final = pd.DataFrame()
    huizu_ins = HzRiskModel(account['rsa_prv_key'],
                            account['aes_key'],
                            account['company_uuid'],
                            url=HzRiskModel.urls[env])
    for _, series in input_excel.iterrows():
        name, phone, idcard = str(series['name']), str(series['phone']), str(series['idcard'])
        print(f'It is running params: name {name}, phone {phone}, idcard {idcard}')
        copied = discard_params.copy()
        copied.extend((name, phone, idcard))
        if len(set(copied)) != len(copied):
            print(f'invalid params name {name}, phone {phone}, idcard {idcard}')
            continue
        ins_result = huizu_ins.set_biz_data(name, str(phone), str(idcard)).request()
        if not huizu_ins.is_succeed():
            if raise_failed_exception:
                raise RequestFailed(f'{ins_result}')
            print('error ins_result  ----> ', ins_result)
            with open(error_log, mode='a') as f:
                print(f"param is {series['idcard']} {series['name']} {series['phone']}")
                f.write(f"{series['idcard']},{series['name']},{series['phone']}\n")
        df_parsed = template_excel.parse(ins_result, index=[0])
        df_final = df_final.append(df_parsed, ignore_index=True)
    Json2Excel.to_excel(df_final, output_file, columns=columns)


if __name__ == '__main__':
    from gitignore import BOOLDATA_ACCOUNT

    file = '/Users/yutou/shouxin/sxProject/pysubway/pysubway/gitignore/haosen.csv'
    output = '/Users/yutou/shouxin/sxProject/pysubway/pysubway/gitignore/haosen_result.xlsx'
    # 有表头
    batch_test(file, BOOLDATA_ACCOUNT, output, skiprows=None, names=('name', 'phone', 'idcard'), header=0)
    # 无表头



from typing import Dict, Any

import pandas as pd

try:
    from service.booldata.base import BoolDataBase
    from utils import udict
except (ImportError, ModuleNotFoundError) as e:
    from pysubway.service.booldata.base import BoolDataBase
    from pysubway.utils import udict


class Network3Eles(BoolDataBase):
    service = 'network_operator_triple_elements'
    mode = 'mode_triple_elements'
    default_part = {
        'name': '',
        'phone': '',
        'idcard': ''
    }

    @classmethod
    def generate_biz_data(cls, name: str, phone: str, ident_number: str) -> Dict[str, Any]:
        biz_data = udict.Dict.filter(locals())
        biz_data.update({
            "service": cls.service,
            "mode": cls.mode
        })
        return biz_data

    def set_biz_data(self, name: str, phone: str, ident_number: str) -> 'Network3Eles':
        self.biz_data = self.generate_biz_data(name, phone, ident_number)
        return self

    def result_as_df_partial(self, append_biz_data: bool = True) -> pd.DataFrame:
        return self.result_as_df(self.response.get('resp_data', {}), self.default_part,
                                 append_biz_data=append_biz_data)


if __name__ == '__main__':
    from gitignore import BOOLDATA_ACCOUNT, booldata_personal_law

    print('test result_as_df_partial start ...')
    data = booldata_personal_law(3)
    name = data['name']
    phone = data['phone']
    idcard = data['id_num']
    law_list = Network3Eles(BOOLDATA_ACCOUNT['rsa_prv_key'], BOOLDATA_ACCOUNT['aes_key'],
                            BOOLDATA_ACCOUNT['company_uuid'])
    law_list.set_biz_data(name, phone, idcard).request()
    s = law_list.result_as_df_partial()
    print(s)
    print('test result_as_df_partial end ...')



from typing import Callable, Dict, Any, Tuple, Union, List

from flask import Flask
from flask import render_template
from flask import request
from flask import send_file
from flask_restful import Resource
from werkzeug.datastructures import ImmutableMultiDict

try:
    from utils.file import File
    from service.authentication import AuthenticationFactory
    from controller.request import Request
    from errors import IncomingDataValueError
    from utils.package import Package
except (ModuleNotFoundError, ImportError) as e:
    from pysubway.utils.file import File
    from pysubway.service.authentication import AuthenticationFactory
    from pysubway.controller.request import Request
    from pysubway.errors import IncomingDataValueError
    from pysubway.utils.package import Package

__all__ = [
    'ViewRestful', 'View'
]


class View:

    @staticmethod
    def new_app_this_dir(dirname: str) -> Flask:
        """

        :param dirname: example File.this_dir(__file__)}/templates
        :return:
        """
        return Flask(__name__, template_folder=dirname)

    @staticmethod
    def resp_template(template: str = 'index.html', **kwargs: Any) -> str:
        """
        view function
        :return index.html
        """
        return render_template(template, **kwargs)

    @staticmethod
    def resp_file(sent_file: str) -> str:
        """
        view function
        :param sent_file: the file will be sent.
        :param authenticate: authenticate account and password
        :return:
        """
        return send_file(sent_file)

    @staticmethod
    def set_template_folder(full_path_dirname: str = '') -> str:
        """
        declare template_folder
        :param full_path_dirname: 
        :return: 
        """
        if full_path_dirname:
            return full_path_dirname
        dirname: str = full_path_dirname or File.this_dir(__file__)
        return File.join_path(dirname, 'templates')

    @staticmethod
    def authenticate(style: str = 'file', **kwargs: Any) -> Callable[[str, str], bool]:
        """
        warp AuthenticationFactory
        :param style: call AuthenticationFile
        :param kwargs: called class construction method params
        :return: callable object, params are (account, password) and returned value is bool
        """
        return AuthenticationFactory(style)(**kwargs).is_authenticated

    @classmethod
    def register_router(cls, app: Flask, router: Dict[Tuple[str, Callable], Dict]) -> None:
        """

        :param app:
        :param router: key is a Tuple: url and view function; value is other options
        :return:
        """
        for basic, options in router.items():
            url, view_func = basic
            app.add_url_rule(url, view_func=view_func, **options)

    @staticmethod
    def request_form(*keys: str) -> Union[ImmutableMultiDict, List[Any]]:
        """
        front end config:
        action: request url
        name: name is necessary, name is ImmutableMultiDict item key
        example:
        <form role="form" action="/download-file/"
                  target="_self"
                  accept-charset="UTF-8"
                  method="POST"
                  autocomplete="off"
                  enctype="application/x-www-form-urlencoded">

            <textarea name="biz_data"></textarea>
            <input ..../>
        </form>
        :return:
        """
        return [request.form.get(key) for key in keys] if keys else request.form


class ViewRestful(Resource):
    request: Request = Request()
    service: Dict = dict()

    @classmethod
    def set_service(cls, package_path: str, prefix: str) -> Dict[str, Any]:
        return Package(package_path).get_classes(prefix)

    def exec_service(self) -> Dict[str, Any]:
        if not self.service:
            pass
        cls_name, method_name, _ = self.request.service
        exec_method = getattr(self.service.get(cls_name), method_name)
        if not exec_method:
            raise IncomingDataValueError(f'class cls_name {cls_name} do not have method {method_name}')
        return exec_method(self.request.data)

    def get(self) -> str:
        return 'hello pysubway ...'
